#region Copyright Syncfusion Inc. 2001 - 2005
//
//  Copyright Syncfusion Inc. 2001 - 2005. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
//
#endregion

using System;
using System.Drawing;
using System.ComponentModel;
using System.Runtime.Serialization;
using System.Drawing.Drawing2D;

using Syncfusion.Windows.Forms.Diagram;

namespace Syncfusion.Windows.Forms.Diagram.Samples.DynamicSymbol
{
	/// <summary>
	/// Summary description for MySymbol.
	/// </summary>
	[Serializable()]
	public class MySymbol : Symbol
	{
		private Syncfusion.Windows.Forms.Diagram.Rectangle outerRect = null;
//		private Ellipse innerEllipse = null;
//		private int curEllipseColor = 0;
		private string strDescription = "Custom Symbol";	

		static System.Drawing.Color[] ellipseColors =
		{
			Color.AliceBlue,
			Color.DeepSkyBlue,
			Color.LightBlue,
			Color.Aquamarine
		};

		public MySymbol()
		{
			//////////////////////////////////////////////////////////////////
			// Add child nodes to the symbol programmatically
			//////////////////////////////////////////////////////////////////

			// Add an outer rectangle
			this.outerRect = new Syncfusion.Windows.Forms.Diagram.Rectangle(0, 0, 120, 80);
			this.outerRect.Name = "Rectangle1";
			this.outerRect.FillStyle.Type = Syncfusion.Windows.Forms.Diagram.FillStyleType.LinearGradient;
			this.outerRect.FillStyle.GradientStartColor = System.Drawing.Color.Orange;
			this.outerRect.FillStyle.Color = Color.Orange;
			this.outerRect.ShadowStyle.Visible = true;
			this.AppendChild(outerRect);

//			// Add an inner ellipse
//			this.innerEllipse = new Ellipse(10, 10, 100, 60);
//			this.innerEllipse.Name = "Ellipse1";
//			this.innerEllipse.FillStyle.Color = ellipseColors[this.curEllipseColor];
//			this.AppendChild(innerEllipse);

//			Label lbl = this.AddLabel("Hello World", BoxPosition.Center);
//			lbl.BackgroundStyle.Color = Color.Transparent;
//			lbl.ReadOnly = true;

			TextNode txtnode = new TextNode("Hello World");
			txtnode.Name = "TextNode1";
			txtnode.Location = new PointF(20,20);
			txtnode.Size = new SizeF(this.outerRect.Width-40, 20);
			this.AppendChild(txtnode);									
		}

		/// <summary>
		/// Serialization constructor for symbols.
		/// </summary>
		/// <param name="info">Serialization state information</param>
		/// <param name="context">Streaming context information</param>
		protected MySymbol(SerializationInfo info, StreamingContext context) : base(info, context)
		{
			this.strDescription = info.GetString("strDescription");
		}

		/// <summary>
		/// This property provides an example of adding a member variable to a custom symbol
		/// and demonstrates how to handle serialization for this property.
		/// </summary>
		[
		Browsable(true),
		Category("General"),
		Description("Description of the symbol"),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)
		]
		public string Description
		{
			get
			{
				return this.strDescription;
			}
			set
			{
				this.strDescription = value;
			}
		}

//		protected override void OnMouseEnter(NodeMouseEventArgs evtArgs)
//		{
//			base.OnMouseEnter(evtArgs);
//			this.outerRect.FillStyle.Color = Color.OrangeRed;
//		}
//
//		protected override void OnMouseLeave(NodeMouseEventArgs evtArgs)
//		{
//			base.OnMouseLeave(evtArgs);
//			this.outerRect.FillStyle.Color = Color.DarkOrange;
//		}
//
//
//		protected override void OnClick(NodeMouseEventArgs evtArgs)
//		{
//			if (this.curEllipseColor == ellipseColors.Length-1)
//			{
//				this.curEllipseColor = 0;
//			}
//			else
//			{
//				this.curEllipseColor++;
//			}
//			this.innerEllipse.FillStyle.Color = ellipseColors[this.curEllipseColor];
//		}

		/// <summary>
		/// Populates a SerializationInfo with the data needed to
		/// serialize the target object.
		/// </summary>
		/// <param name="info">SerializationInfo object to populate</param>
		/// <param name="context">Destination streaming context</param>
		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);

			// Additional member variables are serialized here
			info.AddValue("strDescription", this.strDescription);
		}

		/// <summary>
		/// Called when deserialization is complete.
		/// </summary>
		/// <param name="sender">Object performing the deserialization</param>
		protected override void OnDeserialization(object sender)
		{
			base.OnDeserialization(sender);

			// Get references to the child objects
			this.outerRect = this.GetChildByName("Rectangle1") as Syncfusion.Windows.Forms.Diagram.Rectangle;
			//this.innerEllipse = this.GetChildByName("Ellipse1") as Ellipse;			
		}	

		private bool bInBoundsChanged = false;
		private Matrix startXForm = null;
		private RectangleF startBounds = RectangleF.Empty;

		// Resizing a symbol will effect a scaling of the symbol's transformation matrix. 
		// Instead of scaling the symbol, we restore its transformation matrix to the original scale, and 
		// resize each node in the symbol for the new bounds. This lets us adjust the size of the TextNode 
		// in response to changes in the Symbol bounds.
		protected override void OnBoundsChanged(BoundsEventArgs evtArgs)
		{
			base.OnBoundsChanged (evtArgs);

			if(this.bInBoundsChanged == false)
			{
				this.bInBoundsChanged = true;
				
				// Store the symbol's original transformation matrix and bounds
				if(this.startXForm == null)
					this.startXForm = this.LocalTransform;
				if(this.startBounds == RectangleF.Empty)
					this.startBounds = this.Bounds;				

				// Reset the symbol's transformation matrix to the original scale
				RectangleF currentbounds = this.Bounds;
				Matrix currentxform = this.LocalTransform;
				float[] xform = this.startXForm.Elements;
				this.LocalTransform = new Matrix(xform[0], xform[1], xform[2], xform[3], currentxform.OffsetX, currentxform.OffsetY);

				// Get the dimensional change
				float deltacx = currentbounds.Width - this.startBounds.Width;
				float deltacy = currentbounds.Height - this.startBounds.Height;

				// Apply the dimensional change to each of the nodes in the Symbol
				Syncfusion.Windows.Forms.Diagram.Rectangle rectnode = this.GetChildByName("Rectangle1") as Syncfusion.Windows.Forms.Diagram.Rectangle;
				rectnode.Width += deltacx;
				rectnode.Height += deltacy;
				
				Syncfusion.Windows.Forms.Diagram.TextNode txtnode = this.GetChildByName("TextNode1") as Syncfusion.Windows.Forms.Diagram.TextNode;
				txtnode.Width = rectnode.Width - 40;

				this.startBounds = this.Bounds;
	
				this.bInBoundsChanged = false;
			}
		}

	}
}
